@@ -1,5 +1,5 @@
 package Mail::DMARC::PurePerl;
-our $VERSION = '1.20140208'; # VERSION
+our $VERSION = '1.20140210'; # VERSION
 use strict;
 use warnings;
 
@@ -26,11 +26,11 @@ sub validate {
     $self->result->disposition('none');   # defaults
 
 # 11.2.1 Extract RFC5322.From domain
-    my $from_dom = $self->get_from_dom() or return;
+    my $from_dom = $self->get_from_dom() or return $self->result;
 # 9.6. reject email if the domain appears to not exist
-    $self->exists_in_dns() or return;
+    $self->exists_in_dns() or return $self->result;
     $policy ||= $self->discover_policy();  # 11.2.2 Query DNS for DMARC policy
-    $policy or return;
+    $policy or return $self->result;
 
     #   3.5 Out of Scope  DMARC has no "short-circuit" provision, such as
     #         specifying that a pass from one authentication test allows one
@@ -39,7 +39,7 @@ sub validate {
     $self->is_dkim_aligned;   # 11.2.3. DKIM signature verification checks
     $self->is_spf_aligned;    # 11.2.4. SPF validation checks
     $self->is_aligned()       # 11.2.5. identifier alignment checks
-        and return 1;
+        and return $self->result;
 
     my $effective_p
         = $self->is_subdomain && defined $policy->sp
@@ -50,24 +50,24 @@ sub validate {
     #        disposed of in accordance with the discovered DMARC policy of the
     #        Domain Owner.  See Section 6.2 for details.
     if ( lc $effective_p eq 'none' ) {
-        return;
+        return $self->result;
     }
 
-    return if $self->is_whitelisted;
+    return $self->result if $self->is_whitelisted;
 
     # 7.1.  Policy Fallback Mechanism
     # If the "pct" tag is present in a policy record, application of policy
     # is done on a selective basis.
     if ( !defined $policy->pct ) {
         $self->result->disposition($effective_p);
-        return;
+        return $self->result;
     }
 
     # The stated percentage of messages that fail the DMARC test MUST be
     # subjected to whatever policy is selected by the "p" or "sp" tag
     if ( int( rand(100) ) >= $policy->pct ) {
         $self->result->reason( type => 'sampled_out' );
-        return;
+        return $self->result;
     }
 
     # Those that are not thus selected MUST instead be subjected to the next
@@ -75,7 +75,7 @@ sub validate {
     # the policies are "reject", "quarantine", and "none".
     $self->result->disposition(
         ( $effective_p eq 'reject' ) ? 'quarantine' : 'none' );
-    return;
+    return $self->result;
 }
 
 sub discover_policy {
@@ -159,7 +159,7 @@ sub is_dkim_aligned {
     my $self = shift;
 
     $self->result->dkim('fail');    # our 'default' result
-    my $pass_sigs = $self->get_dkim_pass_sigs() or return;
+    $self->get_dkim_pass_sigs() or return;
 
     # 11.2.3 Perform DKIM signature verification checks.  A single email may
     #        contain multiple DKIM signatures.  The results MUST include the
@@ -182,8 +182,7 @@ sub is_dkim_aligned {
             identity => '',                      # TODO, what is this?
         };
 
-        if ( $dkim_dom eq $from_dom )
-        {    # strict alignment requires exact match
+        if ( $dkim_dom eq $from_dom ) { # strict alignment requires exact match
             $self->result->dkim('pass');
             $self->result->dkim_align('strict');
             $self->result->dkim_meta($dkmeta);
@@ -191,7 +190,7 @@ sub is_dkim_aligned {
         }
 
         # don't try relaxed if policy specifies strict
-        next if $policy->adkim && lc $policy->adkim eq 's';
+        next if $policy->adkim && 's' eq lc $policy->adkim;
 
         # don't try relaxed if we already got a strict match
         next if 'pass' eq $self->result->dkim;
@@ -204,32 +203,35 @@ sub is_dkim_aligned {
             $self->result->dkim_meta($dkmeta);
         }
     }
-    return 1 if 'pass' eq $self->result->dkim;
+    return 1 if 'pass' eq lc $self->result->dkim;
     return;
 }
 
 sub is_spf_aligned {
     my $self    = shift;
     my $spf_dom = shift;
-    if ( !$spf_dom && !$self->spf ) { croak "missing SPF!"; }
 
-    if ( ! $spf_dom ) {
-        foreach my $spf ( @{ $self->spf } ) {
-            next if ! $spf->{domain};
-            $spf_dom = $spf->{domain};
-            last;
+    if ( !$spf_dom && !$self->spf ) { croak "missing SPF!"; }
+    if ( !$spf_dom ) {
+        my @passes = grep { $_->{result} =~ /pass/i } @{ $self->spf };
+        if (scalar @passes == 0) {
+            $self->result->spf('fail');
+            return 0;
         };
+        my ($ref)  = grep { $_->{scope} && $_->{scope} eq 'mfrom' } @passes;
+        if (!$ref) {
+            ($ref) = grep { $_->{scope} && $_->{scope} eq 'helo' } @passes;
+        }
+        if (!$ref) { ($ref) = $passes[0]; };
+        $spf_dom = $ref->{domain};
     };
-    $spf_dom or croak "missing SPF domain";
 
     # 11.2.4 Perform SPF validation checks.  The results of this step
     #        MUST include the domain name from the RFC5321.MailFrom if SPF
     #        evaluation returned a "pass" result.
 
-    if ( !$spf_dom ) {
-        $self->result->spf('fail');
-        return 0;
-    }
+    $self->result->spf('fail');
+    return 0 if !$spf_dom;
 
     my $from_dom = $self->header_from or croak "header_from not set!";
 
@@ -240,19 +242,17 @@ sub is_spf_aligned {
     }
 
     # don't try relaxed match if strict policy requested
-    if ( $self->policy->aspf && lc $self->policy->aspf eq 's' ) {
-        $self->result->spf('fail');
+    if ( $self->policy->aspf && 's' eq lc $self->policy->aspf ) {
         return 0;
     }
 
     if ( $self->get_organizational_domain($spf_dom) eq
-        $self->get_organizational_domain($from_dom) )
+         $self->get_organizational_domain($from_dom) )
     {
         $self->result->spf('pass');
         $self->result->spf_align('relaxed');
         return 1;
     }
-    $self->result->spf('fail');
     return 0;
 }
 
@@ -303,7 +303,7 @@ sub get_dkim_pass_sigs {
         croak "dkim needs to be an array reference!";
     }
 
-    return grep { $_->{result} eq 'pass' } @$dkim_sigs;
+    return grep { 'pass' eq lc $_->{result} } @$dkim_sigs;
 }
 
 sub get_organizational_domain {
@@ -543,7 +543,7 @@ Mail::DMARC::PurePerl - Pure Perl implementation of DMARC
 
 =head1 VERSION
 
-version 1.20140208
+version 1.20140210
 
 =head1 METHODS
 
